/*
 *  stereo -- Stereo matching software
 *  $Id: image.c 197 2004-07-20 16:07:20Z eric $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#include <stdio.h>
#include <stdint.h>

#include <cv.h>
#include <highgui.h>

#include "defs.h"
#include "image.h"

int image_new(image_t *image, unsigned int width, unsigned int height, unsigned int depth)
{
    IplImage *tmp;

    image->width = width;
    image->height = height;
    image->pitch = (width * depth + 15) & ~15;
    image->depth = depth;

# if 0
    image->pixels = malloc(height * image->pitch);
    if(!image->pixels) return -1;

    image->_type = IMAGE_NORMAL;
#else
    tmp = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, depth);

    image->width = tmp->width;
    image->height = tmp->height;
    image->pitch = tmp->widthStep;
    image->depth = tmp->nChannels;
    image->pixels = tmp->imageData;

    image->_private = (void *)tmp;
    image->_type = IMAGE_OPENCV;
#endif

    return 0;
}

int image_load(image_t *image, char *filename)
{
    IplImage *tmp;

    tmp = (void *)cvLoadImage(filename, -1);
    if(!tmp) return -1;

    if(tmp->nChannels != 3)
    {
        IplImage *color;
        color = cvCreateImage(cvSize(tmp->width, tmp->height), IPL_DEPTH_8U, 3);
        if(!color)
        {
            cvReleaseImage(&tmp);
            return -1;
        }
        cvCvtColor(tmp, color, CV_GRAY2BGR);
        cvReleaseImage(&tmp);
        tmp = color;
    }

    image->width = tmp->width;
    image->height = tmp->height;
    image->pitch = tmp->widthStep;
    image->depth = tmp->nChannels;
    image->pixels = tmp->imageData;

    image->_private = (void *)tmp;
    image->_type = IMAGE_OPENCV;

    return 0;
}

int image_display(image_t *image)
{
    char name[BUFSIZ];
    IplImage *tmp;

    if(image->_type != IMAGE_OPENCV)
        return -1;

    tmp = (IplImage *)image->_private;
    sprintf(name, "Image %p (%i x %i)", tmp, image->width, image->height);
    cvNamedWindow(name, 0);
    cvShowImage(name, tmp);
    cvResizeWindow(name,640,480);

    return 0;
}

int image_free(image_t *image)
{
    switch(image->_type)
    {
        case IMAGE_NORMAL:
            free(image->pixels);
            break;
        case IMAGE_OPENCV:
        {
            IplImage *tmp;
            tmp = (IplImage *)image->_private;
            cvReleaseImage(&tmp);
            break;
        }
        default:
            break;
    }

    image->_type = 0;

    return 0;
}

int getpixel(image_t *image, int x, int y, int *r, int *g, int *b)
{
    if(x < 0 || y < 0 || x >= image->width || y >= image->height)
    {
        *r = 255;
        *g = 0;
        *b = 255;
        return -1;
    }

    *b = image->pixels[y * image->pitch + x * image->depth];
    *g = image->pixels[y * image->pitch + x * image->depth + 1];
    *r = image->pixels[y * image->pitch + x * image->depth + 2];

    return 0;
}

int setpixel(image_t *image, int x, int y, int r, int g, int b)
{
    if(x < 0 || y < 0 || x >= image->width || y >= image->height)
        return -1;

    image->pixels[y * image->pitch + x * image->depth] = b;
    image->pixels[y * image->pitch + x * image->depth + 1] = g;
    image->pixels[y * image->pitch + x * image->depth + 2] = r;

    return 0;
}

int convert_BGR2GRAY(image_t *src,image_t *dst)
{
	image_new(dst,src->width,src->height,1);
	acp(src,dst);
	return 0;
}
